EnableExplicit
EnableASM

CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
  Import ""
    GetTickCount64()
    EndImport
  CompilerEndIf

Procedure PreCalcCode()
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\PreCalcMirrorData.x64"
    IncludeFile "\dev\software\libraries\linespec4\source\PreCalcOptData.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\PreCalcMirrorData.x86"
    IncludeFile "\dev\software\libraries\linespec4\source\PreCalcOptData.x86"
    CompilerEndIf
  EndProcedure
  
ProcedureDLL.i Init(Reserved.i)
  Structure DSCBUFFERDESC 
    dwSize.l            ; Size of the structure, in bytes. This member must be initialized before the structure is used. 
    dwFlags.l           ; Flags specifying the capabilities of the buffer 
    dwBufferBytes.l     ; Size of capture buffer to create, in bytes. 
    dwReserved.l        ; Must be 0 
    *lpwfxFormat        ; Address of a WAVEFORMATEX or WAVEFORMATEXTENSIBLE structure specifying the waveform format for the buffer. 
    EndStructure
  #DS_OK = 0
  #DSCBSTATUS_CAPTURING = 1
  #DSCBLOCK_ENTIREBUFFER = 1
  
  Global HasPreCalced .b
  Global hThread.i
  Global QuitThread.b
  Global *MemRec = ?MemRec
  Global *MemDon = ?MemDon
  Global *MemShw = ?MemShw
  Global LastDon.i
  Global LastShw.i

  Global *IDirectSoundCapture.IDirectSoundCapture
  Global *IDirectSoundCaptureBuffer.IDirectSoundCaptureBuffer
  
  If Not HasPreCalced
    PreCalcCode()
    EndIf
  EndProcedure
  
Procedure.i RecThread(Param.i)
  Define.i xParam1
  Define.i xParam2
  Define.i xLong1
  
Loop0:
  If *IDirectSoundCaptureBuffer\Start(0) = #DS_OK: Goto Loop1: EndIf
  Sleep_(20)
  If QuitThread: Goto Done: EndIf
  Goto Loop0
  
Loop1:
  Sleep_(20)
  If QuitThread: Goto Done: EndIf
  If *IDirectSoundCaptureBuffer\GetStatus(@xParam1) = #DS_OK
    If Not xParam1 & #DSCBSTATUS_CAPTURING: Goto Loop2: EndIf
    EndIf
  Goto Loop1
  
Loop2:
  If *IDirectSoundCaptureBuffer\Stop() = #DS_OK: Goto Loop3: EndIf
  Sleep_(20)
  If QuitThread: Goto Done: EndIf
  Goto Loop2

Loop3:
  If *IDirectSoundCaptureBuffer\Lock(0, 0, @xParam1, @xParam2, 0, 0, #DSCBLOCK_ENTIREBUFFER) = #DS_OK
    CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
      xLong1 = GetTickCount64()
      CompilerElse
      xLong1 = GetTickCount_()
      CompilerEndIf
    If xParam2 < $2000
      *IDirectSoundCaptureBuffer\Unlock(xParam1, xParam2, 0, 0)
      Goto Loop0
      EndIf
    CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
      MOV rax, xParam1
      MOV rcx, *MemRec
      PUSH rsi
      MOV rsi, rax
      PUSH rdi
      MOV rdi, rcx
      MOV rcx, $2000 / $8
      CLD
      REP MOVSQ
      POP rdi
      POP rsi
      CompilerElse
      MOV eax, xParam1
      MOV ecx, *MemRec
      PUSH esi
      MOV esi, eax
      PUSH edi
      MOV edi, ecx
      MOV ecx, $2000 / $4
      CLD
      REP MOVSD
      POP edi
      POP esi
      CompilerEndIf
    *IDirectSoundCaptureBuffer\Unlock(xParam1, xParam2, 0, 0)
    xParam1 = *MemRec
    EnterCriticalSection_(?CritSection)
    xParam2 = *MemDon
    *MemDon = xParam1
    LastDon = xLong1
    LeaveCriticalSection_(?CritSection)
;Debug "FlippedRecDon"
    *MemRec = xParam2
    Goto Loop0
    EndIf
  Sleep_(20)
  If Not QuitThread: Goto Loop3: EndIf
Done:
  EndProcedure
ProcedureDLL.i RecStart(Reserved.i)
  Define.i xLong1
  Define.WAVEFORMATEX xType1
  Define.DSCBUFFERDESC xType2
  
  If DirectSoundCaptureCreate_(0, @*IDirectSoundCapture, 0) = #DS_OK
    With xType1
      \wFormatTag = 1
      \nChannels = 2
      \nSamplesPerSec = 44100
      \nAvgBytesPerSec = 176400
      \nBlockAlign = 4
      \wBitsPerSample = 16
      EndWith
      
    With xType2
      \dwSize = SizeOf(xType2)
      \dwBufferBytes = $2000
      \lpwfxFormat = @xType1
      EndWith
      
    If *IDirectSoundCapture\CreateCaptureBuffer(@xType2, @*IDirectSoundCaptureBuffer, 0) = #DS_OK
      InitializeCriticalSection_(?CritSection)
      hThread = CreateThread_(0, 0, @RecThread(), 1, 0, @xLong1)
      If hThread
        ProcedureReturn 1
        EndIf
      DeleteCriticalSection_(?CritSection)  
      *IDirectSoundCaptureBuffer\Release()
      EndIf
    *IDirectSoundCapture\Release()
    EndIf
  EndProcedure
ProcedureDLL.i RecStop()
  QuitThread = 1
  WaitForSingleObject_(hThread, -1)
  CloseHandle_(hThread)
  DeleteCriticalSection_(?CritSection)  
  *IDirectSoundCaptureBuffer\Release()
  *IDirectSoundCapture\Release()
  EndProcedure
ProcedureDLL.i RecGetPtr(*ptrTimeStamp)
  Define.i xLong1
  Define.i xLong2

  If LastDon
    xLong1 = *MemShw
    EnterCriticalSection_(?CritSection)
    xLong2 = *MemDon
    *MemDon = xLong1
    LastShw = LastDon
    LastDon = 0
    LeaveCriticalSection_(?CritSection)
    *MemShw = xLong2
    If *ptrTimeStamp
      PokeI(*ptrTimeStamp, LastShw)
      EndIf
    ProcedureReturn *MemShw
    EndIf
  ProcedureReturn 0
  EndProcedure

ProcedureDLL.i SetWaveData(*WaveData)
  Define.l xResult
  If *WaveData = 0
    If hThread
      *WaveData = RecGetPtr(@xResult)
      EndIf
    EndIf
  
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\FFT.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\FFT.x86"
    CompilerEndIf
  ProcedureReturn xResult
CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
  IncludeFile "\dev\software\libraries\linespec4\source\FFTStgL.x64"
  IncludeFile "\dev\software\libraries\linespec4\source\FFTStgR.x64"
  CompilerElse
  IncludeFile "\dev\software\libraries\linespec4\source\FFTStgL.x86"
  IncludeFile "\dev\software\libraries\linespec4\source\FFTStgR.x86"
  CompilerEndIf  
EndProcedure

ProcedureDLL.i GetFullSpectrumStereo(*Spectrum)
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\GetFullSpectrum_Stereo.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\GetFullSpectrum_Stereo.x86"
    CompilerEndIf
  EndProcedure
ProcedureDLL.i GetFullSpectrumStereoOpt(*Spectrum)
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\GetFullSpectrum_StereoOpt.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\GetFullSpectrum_StereoOpt.x86"
    CompilerEndIf
  EndProcedure
ProcedureDLL.i GetFullSpectrumMono(*Spectrum)
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\GetFullSpectrum_Mono.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\GetFullSpectrum_Mono.x86"
    CompilerEndIf
  EndProcedure
ProcedureDLL.i GetFullSpectrumMonoOpt(*Spectrum)
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\GetFullSpectrum_MonoOpt.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\GetFullSpectrum_MonoOpt.x86"
    CompilerEndIf
  EndProcedure

ProcedureDLL.i SpectrumMonoMix(*SpectrumIn, Size.i, *SpectrumOut)
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\SpectrumMonoMix.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\SpectrumMonoMix.x86"
    CompilerEndIf
  EndProcedure

ProcedureDLL.i SpectrumOptimizeStereo(*SpectrumIn, Size.i, *SpectrumOut)
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\SpectrumOptimizeStereo.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\SpectrumOptimizeStereo.x86"
    CompilerEndIf
  EndProcedure
ProcedureDLL.i SpectrumOptimizeMono(*SpectrumIn, Size.i, *SpectrumOut)
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
    IncludeFile "\dev\software\libraries\linespec4\source\SpectrumOptimizeMono.x64"
    CompilerElse
    IncludeFile "\dev\software\libraries\linespec4\source\SpectrumOptimizeMono.x86"
    CompilerEndIf
  EndProcedure

ProcedureDLL.i SpectrumCompactStereo(*SpectrumIn, ArraySizeIn.i, *SpectrumOut, ArraySizeOut.i)
  Define.l xLong1
  Define.l xLong2
  Define.l xIdx
  Define.f xValL
  Define.f xValR
  Define.f xValT
  
  If *SpectrumIn = 0
    *SpectrumIn = ?SpecTmp
    EndIf
  If *SpectrumOut = 0
    *SpectrumOut = ?SpecTmp
    EndIf
  
  If ArraySizeOut < ArraySizeIn
    For xLong1 = 0 To ArraySizeIn - 1
      xLong2 = (xLong1 * ArraySizeOut) / ArraySizeIn
      If xLong2 = xIdx
        xValT = PeekF(*SpectrumIn + xLong1 * 4)
        If xValT > xValL
          xValL = xValT
          EndIf
        xValT = PeekF(*SpectrumIn + (ArraySizeIn + xLong1) * 4)
        If xValT > xValR
          xValR = xValT
          EndIf
        Else
        PokeF(*SpectrumOut + xIdx * 4, xValL)
        xValL = PeekF(*SpectrumIn + xLong1 * 4)
        PokeF(*SpectrumOut + (ArraySizeOut + xIdx) * 4, xValR)
        xValR = PeekF(*SpectrumIn + (ArraySizeIn + xLong1) * 4)
        xIdx = xLong2
        EndIf
      Next
    PokeF(*SpectrumOut + (ArraySizeOut - 1) * 4, xValL)
    PokeF(*SpectrumOut + (ArraySizeOut * 2 - 1) * 4, xValR)
    EndIf
  EndProcedure

ProcedureDLL.i SpectrumCompactMono(*SpectrumIn, ArraySizeIn.i, *SpectrumOut, ArraySizeOut.i)
  Define.l xLong1
  Define.l xLong2
  Define.l xIdx
  Define.f xValL
  Define.f xValT
  
  If *SpectrumIn = 0
    *SpectrumIn = ?SpecTmp
    EndIf
  If *SpectrumOut = 0
    *SpectrumOut = ?SpecTmp
    EndIf

  If ArraySizeOut < ArraySizeIn
    For xLong1 = 0 To ArraySizeIn - 1
      xLong2 = (xLong1 * ArraySizeOut) / ArraySizeIn
      If xLong2 = xIdx
        xValT = PeekF(*SpectrumIn + xLong1 * 4)
        If xValT > xValL
          xValL = xValT
          EndIf
        Else
        PokeF(*SpectrumOut + xIdx * 4, xValL)
        xValL = PeekF(*SpectrumIn + xLong1 * 4)
        xIdx = xLong2
        EndIf
      Next
    PokeF(*SpectrumOut + (ArraySizeOut - 1) * 4, xValL)
    EndIf
  EndProcedure

ProcedureDLL.i SpectrumCompactStereoSM(*SpectrumIn, ArraySizeIn.i, *SpectrumOut, ArraySizeOut.i)
  SpectrumCompactMono(*SpectrumIn, ArraySizeIn, *SpectrumOut, ArraySizeOut)
  SpectrumCompactMono(*SpectrumIn + ArraySizeIn * 4, ArraySizeIn, *SpectrumOut + ArraySizeOut * 4, ArraySizeOut)
  EndProcedure

ProcedureDLL.i SpectrumExpandStereo(*SpectrumIn, ArraySizeIn.i, *SpectrumOut, ArraySizeOut.i)
  Define.l xLong1
  Define.l xLong2
  
  If *SpectrumIn = 0
    *SpectrumIn = ?SpecTmp
    EndIf
  If *SpectrumOut = 0
    *SpectrumOut = ?SpecTmp
    EndIf

  If ArraySizeOut > ArraySizeIn
    For xLong1 = ArraySizeOut - 1 To 0 Step -1
      xLong2 = (xLong1 * ArraySizeIn) / ArraySizeOut
      PokeF(*SpectrumOut + xLong1 * 4, PeekF(*SpectrumIn + xLong2 * 4))
      PokeF(*SpectrumOut + (ArraySizeOut + xLong1) * 4, PeekF(*SpectrumIn + (ArraySizeIn + xLong2) * 4))
      Next
    EndIf
  EndProcedure

ProcedureDLL.i SpectrumExpandMono(*SpectrumIn, ArraySizeIn.i, *SpectrumOut, ArraySizeOut.i)
  Define.l xLong1
  Define.l xLong2
  
  If *SpectrumIn = 0
    *SpectrumIn = ?SpecTmp
    EndIf
  If *SpectrumOut = 0
    *SpectrumOut = ?SpecTmp
    EndIf

  If ArraySizeOut > ArraySizeIn
    For xLong1 = ArraySizeOut - 1 To 0 Step -1
      xLong2 = (xLong1 * ArraySizeIn) / ArraySizeOut
      PokeF(*SpectrumOut + xLong1 * 4, PeekF(*SpectrumIn + xLong2 * 4))
      Next
    EndIf
  EndProcedure

ProcedureDLL.i SpectrumExpandStereoSM(*SpectrumIn, ArraySizeIn.i, *SpectrumOut, ArraySizeOut.i)
  SpectrumExpandMono(*SpectrumIn + ArraySizeIn * 4, ArraySizeIn, *SpectrumOut + ArraySizeOut * 4, ArraySizeOut)
  SpectrumExpandMono(*SpectrumIn, ArraySizeIn, *SpectrumOut, ArraySizeOut)
  EndProcedure

DisableASM

DataSection
!StgButterInitReal_0:
!DD $40000000
!StgButterInitImag_0:
!DD $B3BBBD2E
!StgButterInitReal_1:
!DD $3F7FFFFF
!StgButterInitImag_1:
!DD $3F800000
!StgButterInitReal_2:
!DD $3E95F61B
!StgButterInitImag_2:
!DD $3F3504F3
!StgButterInitReal_3:
!DD $3D9BE50C
!StgButterInitImag_3:
!DD $3EC3EF16
!StgButterInitReal_4:
!DD $3C9D6830
!StgButterInitImag_4:
!DD $3E47C5C2
!StgButterInitReal_5:
!DD $3B9DC971
!StgButterInitImag_5:
!DD $3DC8BD36
!StgButterInitReal_6:
!DD $3A9DE1C9
!StgButterInitImag_6:
!DD $3D48FB30
!StgButterInitReal_7:
!DD $399DE7DF
!StgButterInitImag_7:
!DD $3CC90AB0
!StgButterInitReal_8:
!DD $389DE965
!StgButterInitImag_8:
!DD $3C490E90
!StgButterInitReal_9:
!DD $379DE9C6
!StgButterInitImag_9:
!DD $3BC90F88
!StgButterInitReal_A:
!DD $369DE9DF
!StgButterInitImag_A:
!DD $3B490FC6
!Flt_2048 DD 2048.0
!Flt_1_2 DD 0.5
!Flt_1_8000 DD 0.000125
!Flt_1_16000 DD 0.0000625
;!Flt_30_8000 DD 0.00375
;!Flt_30_16000 DD 0.001875
!Flt_30 DD 30.0
!Flt_60 DD 60.0
!Flt_240000 DD 240000.0
!Flt_480000 DD 480000.0
!Flt_2_3 DD 0.666666666666666666666666666666



!Flt_1_800PI DD $9217271A, $3F7015BF

CritSection: ;24 (dec)
!  DQ 0, 0, 0, 0, 0, 0
CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
!  DD 0, 0, 0, 0, 0, 0
CompilerEndIf

!hCachedAndPrepared:
!RB $1000
!hCachedAndPrepared2:
!RB $1000
!hCachedAndPrepared3:
!ArraySize:
!RD 1

MemRec:
!RB $2000
MemDon:
!RB $2000
MemShw:
!RB $2000
SpecTmp:
!SpecTmp:
!RB $2000
!RIValues:
!RB $2000
!RIValues_Post:
!RB $2000
!RIValues2:
!RB $4000
!RIValues_Post2:
!RB $4000
!BoostValues30:
  !RB $1000
;  !BoostValues60:
;  !RB $1000
  !BoostValues240000:
  !RB $1000
  !BoostValues480000:
  !RB $1000
!MirrorData:
!RW 2050

CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
CompilerElse
!Staging_xLong1:
!Arraying_xLong1:
!RD 1
!Staging_xLong2:
!Arraying_xLong2:
!RD 1
!Staging_StageSize:
!Arraying_xLong3:
!RD 1
CompilerEndIf
EndDataSection

Debug Str(30/8000)
Debug Str(30/8000/2)

; IDE Options = PureBasic 4.61 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 199
; FirstLine = 168
; Folding = u---
; EnableAsm
; Executable = ..\lnspec32.dll
; DisableDebugger
; AddResource = Version32.rc